<?php

/**
 * Load oAuth2Cleint and Exception library
 */
require_once __DIR__ . "/OAuth2Client.inc";
require_once __DIR__ . "/OAuth2Exception.inc";


/**
 * OAuth2Consumer Class
 *
 * @package labs.raynux.com
 * @author  Md. Rayhan Chowdhury <ray@raynux.com>
 * @license MIT
 */
class OAuth2Consumer extends OAuth2Client {

    /**
     * Configuration data
     * 
     * @var array
     */
    private $_configs = array(
                        'base_uri' => '',
                        'authorize_uri' => 'authorize',
                        'access_token_uri' => 'token',
                        'file_upload_support' => false,
                        'cookie_support' => false,
                        'scope' => '',
                        'redirect_uri' => '',
                        'oauth_token_name' => 'access_token' // Draft 10 says it should be oauth_token, but google, facebook and microsoft support access_token
                    );

    /**
     * Instances of services
     * 
     * @var array
     * @access private
     */
    private static $__instances = array();

    /**
     * Class Constror
     *
     * @param array $config
     */
    function  __construct($config = array()) {
        $config = array_merge($this->_configs, $config);
        parent::__construct($config);
    }

    /**
     * Get Instance
     * 
     * @param string $provider
     * @param array $configs
     * @return object
     */
    public static function &getInstance($provider, $configs = null) {
        if (!isset(self::$__instances[$provider]) || is_null(self::$__instances[$provider])) {
            self::$__instances[$provider] = new self($configs);
        }

        return self::$__instances[$provider];
    }

    /**
     * Override makeOAuth2Request
     *
     * To fixe problem with api call to Google, Facebook and Windows Live
     */
    protected function makeOAuth2Request($path, $method = 'GET', $params = array()) {
        if (empty($params[$this->_configs['oauth_token_name']])) {
            if ($this->getVariable($this->_configs['oauth_token_name'])) {
                $params[$this->_configs['oauth_token_name']] = $this->getVariable($this->_configs['oauth_token_name']);
            } else if ($oauth_token = $this->getAccessToken()) {
                $params[$this->_configs['oauth_token_name']] = $oauth_token;
            }
        }
        return $this->makeRequest($path, $method, $params);
    }

    /**
     * Get Authorizaton End Point URL
     * 
     * @return string
     */
    public function getAuthUrl($extraParams = array()) {
        $params = array(
                            'client_id' => $this->getVariable('client_id'),
                            'response_type' => 'code',
                            'scope' => $this->getVariable('scope'),
                            'redirect_uri' => $this->getVariable('redirect_uri')
                        );
        if (!empty($extraParams)) {
            $params = array_merge($params, $extraParams);
        }
        return $this->getVariable('authorize_uri') . '?' . http_build_query($params);
    }

    /**
     * Redirect to OAuth2 end point for authorization
     * 
     * @param array $extraParams
     */
    public function authorize($extraParams = array()) {
        header('Location: ' . $this->getAuthUrl());
    }

}

?>